
/*
  CEOI 2006
  Competition Day #1
  Task WALK
  Output correctness checker
*/

#include <algorithm>
#include <cstdarg>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <set>
#include <vector>

using namespace std;

#define M_LIMIT 1000000

int N, M;
long long official_length, your_length;
int targetx, targety;

struct segment {
   int x1, y1, x2, y2;
   segment( int x1, int y1, int x2, int y2 ) : x1(x1), y1(y1), x2(x2), y2(y2) { }
};

struct event {
   int where; char type; int who;
   event( int where, int type, int who ) : where(where), type(type), who(who) { }
};
bool operator < ( const event &A, const event &B ) {
   if ( A.where != B.where ) return A.where < B.where;
   if ( A.type != B.type ) return A.type < B.type;
   if ( A.who != B.who ) return A.who < B.who;
   return 0;
}

int mysgn( int x ) { return x == 0 ? 0 : x < 0 ? -1 : 1; }

int main(int argc, char *argv[])
{
   if ( argc <= 3 ) {
      fprintf( stderr, "usage: check input-file official-output-file your-output-file\n" );
      return 2;
   }

   FILE
      *fin  = fopen(argv[1], "rt"),
      *foff = fopen(argv[2], "rt"),
      *fout = fopen(argv[3], "rt");

   if ( fin == NULL ) {
      fprintf( stderr, "Error opening input file (\"%s\") for reading!\n", argv[1] );
      return 2;
   }

   if ( foff == NULL ) {
      fprintf( stderr, "Error opening official output file (\"%s\") for reading!\n", argv[2] );
      return 2;
   }

   if ( fout == NULL ) {
      fprintf( stderr, "Error opening your output file (\"%s\") for reading!\n", argv[3] );
      return 1;
   }

   if ( fscanf( foff, "%lld", &official_length ) != 1 ) {
      fprintf( stderr, "Error reading official output (length)!\n" );
      return 2;
   }

   if ( fscanf( fout, "%lld", &your_length ) != 1 ) {
      fprintf( stderr, "Error reading your output (path length).\n" );
      return 2;
   }

   if ( your_length != official_length ) {
      printf( "Found length %lld, official output says %lld.\n", your_length, official_length );
      return 1;
   }

   if ( M > M_LIMIT ) {
      printf( "M is too large (more than %d)!\n", M_LIMIT );
      return 1;
   }

   if ( fscanf( fout, "%d", &M ) != 1 ) {
      printf( "Partially correct (80%% score). Error reading segment length.\n" );
      return 0;
   }

   vector<segment> D;

   if ( fscanf( fin, "%d%d", &targetx, &targety ) != 2 ) {
      fprintf( stderr, "Error reading input (destination)!\n" );
      return 2;
   }

   if ( fscanf( fin, "%d", &N ) != 1 ) {
      fprintf( stderr, "Error reading input (N)!\n" );
      return 2;
   }

   for ( int i=0; i<N; ++i ) {
      int x1, y1, x2, y2;
      if ( fscanf( fin, "%d%d%d%d", &x1, &y1, &x2, &y2 ) != 4 ) {
         fprintf( stderr, "Error reading input (N)!\n" );
         return 2;
      }
      if ( x1 > x2 ) swap( x1, x2 );
      if ( y1 > y2 ) swap( y1, y2 );

      D.push_back( segment( x1, y1, x2, y1 ) );
      if ( y1 < y2 ) {
         D.push_back( segment( x1, y2, x2, y2 ) );
         if ( y1+1 < y2 ) {
            D.push_back( segment( x1, y1+1, x1, y2-1 ) );
            if ( x1 < x2 ) {
               D.push_back( segment( x2, y1+1, x2, y2-1 ) );
            }
         }
      }
   }

   int x = 0, y = 0;
   long long sum = 0;
   for ( int i=0; i<M; ++i ) {
      int dx, dy;
      if ( fscanf( fout, "%d%d", &dx, &dy ) != 2 ) {
         printf( "Partially correct (80%% score). Error reading %d-th segment of the path!\n", i+1 );
         return 0;
      }

      sum += abs(dx) + abs(dy);
      D.push_back( segment( x+mysgn(dx), y+mysgn(dy), x+dx, y+dy ) );

      x += dx;
      y += dy;

      if ( sum > your_length ) {
         printf( "Partially correct (80%% score). Path is not of advertised length!\n" );
         return 0;
      }
   }

   if ( x != targetx || y != targety ) {
      printf( "Partially correct (80%% score). Path ends at the wrong point.\n" );
      return 0;
   }

   vector<event> E;
   for ( int i = 0; i < (int)D.size(); ++i ) {
      if ( D[i].x1 == D[i].x2 )
         E.push_back( event( D[i].x1, 2, i ) );
      else {
         E.push_back( event( D[i].x1, 1, i ) );
         E.push_back( event( D[i].x2, 3, i ) );
      }

   }

   sort( E.begin(), E.end() );

   set<int> aktivni;
   for ( vector<event>::iterator it = E.begin(); it != E.end(); ++it ) {
      if ( it->type == 1 ) {
         if ( !aktivni.insert( D[it->who].y1 ).second ) {
            printf( "Partially correct (80%% score). Path intersects a building.\n" );
            return 0;
         }
      }
      if ( it->type == 2 ) {
         set<int>::iterator x = aktivni.lower_bound( D[it->who].y1 );
         if ( x != aktivni.end() && *x <= D[it->who].y2 ) {
            printf( "Partially correct (80%% score). Path intersects a building.\n" );
            return 0;
         }
      }
      if ( it->type == 3 ) {
         aktivni.erase( D[it->who].y1 );
      }
   }

   fclose(fin);
   fclose(foff);
   fclose(fout);

   printf( "Correct!\n" );
   return 0;
}
