
(*
  CEOI 2006
  Competition Day #2
  Task MEANDIAN
  Sample library for Pascal
*)

(*
  input file format:
  N
  a1 a2 ... aN
*)

unit libmean;

interface

function  Init : longint;
function  Meandian(a, b, c, d : longint) : longint;
procedure Solution(var sol : array of longint);

implementation

const LOG=true;
const QUERY_LIMIT = 1000;

const
    initialized : boolean = false;

var flog    : text;
    N       : longint;
    arr     : array[0..99] of longint;
    queries : longint;
    cant    : array[0..3] of longint;
    temp    : array[1..100000] of boolean;

procedure die(msg1, msg2, msg3, msg4 : string; p1, p2, p3 : longint);
begin
    if LOG then
    begin
        append(flog);
        if msg4<>'' then writeln(flog, msg1, p1, msg2, p2, msg3, p3, msg4) else
        if msg3<>'' then writeln(flog, msg1, p1, msg2, p2, msg3) else
        if msg2<>'' then writeln(flog, msg1, p1, msg2) else
        writeln(flog, msg1);
        close(flog);
        if msg4<>'' then writeln(msg1, p1, msg2, p2, msg3, p3, msg4) else
        if msg3<>'' then writeln(msg1, p1, msg2, p2, msg3) else
        if msg2<>'' then writeln(msg1, p1, msg2) else
        writeln(msg1);
    end else
    begin
        if msg4<>'' then writeln(msg1, p1, msg2, p2, msg3, p3, msg4) else
        if msg3<>'' then writeln(msg1, p1, msg2, p2, msg3) else
        if msg2<>'' then writeln(msg1, p1, msg2) else
        writeln(msg1);
    end;
    halt(1);
end;

function Init : longint;
var i : longint;
begin
    if initialized then
        die('Init called more than once', '', '', '', 0, 0, 0);

    N := 0;
    queries := 0;

    if LOG then
    begin
        assign(flog, 'meandian.log');
        rewrite(flog);
        write(flog, 'Init => ');
        close(flog);
    end;

    {$i-}
    readln(N);
    {$i+}
    if ioresult<>0 then
        die( 'Error reading N', '', '', '', 0, 0, 0);

    if ( N < 4 ) or ( N > 100 ) then
        die( 'N must be between 4 and 100', '', '', '', 0, 0, 0);

    for i:=0 to N-1 do
    begin
        {$i-}
        read(arr[i]);
        {$i+}
        if ioresult<>0 then
            die( 'Error reading input', '', '', '', 0, 0, 0);
        if arr[i] mod 2 <> 0 then
            die( 'Number ',' (at position ',') should be even', '', arr[i], i+1, 0);
         if (arr[i] <= 0) or (arr[i] > 100000 ) then
            die( 'Number ',' (at position ',') should be a positive integer <= 100000', '', arr[i], i+1, 0);
    end;

    cant[0]:=+1000000000;
    cant[1]:=+1000000000;
    cant[2]:=-1000000000;
    cant[3]:=-1000000000;
    for i:=1 to 100000 do temp[i]:=false;
    for i:=0 to N-1 do
    begin
        if temp[arr[i]] then
            die( 'Duplicates found (number ', ')', '', '' , arr[i], 0, 0);
        temp[arr[i]]:=true;
        if arr[i]<cant[1] then
        begin
            cant[1]:=arr[i];
            if arr[i]<cant[0] then
            begin
                cant[1]:=cant[0];
                cant[0]:=arr[i];
            end;
        end;
        if arr[i]>cant[3] then
        begin
            cant[3]:=arr[i];
            if arr[i]>cant[2] then
            begin
                cant[3]:=cant[2];
                cant[2]:=arr[i];
            end;
        end;
    end;

    if LOG then
    begin
        append(flog);
        writeln(flog, 'OK (N=',N,')');
        close(flog);
    end;

    initialized := true;

    Init := N;
end;

procedure compare_swap_ints(var a, b : longint);
var t : longint;
begin
    if arr[a-1] > arr[b-1] then
    begin
        t := a;
        a := b;
        b := t;
    end;
end;

function Meandian(a, b, c, d : longint) : longint;
var ret : longint;
begin
    if LOG then
    begin
        append(flog);
        write(flog, 'Meandian(', a, ', ', b, ', ', c, ', ', d, ') => ');
        close(flog);
    end;

    inc (queries);

    if queries > QUERY_LIMIT then
        die( 'Too many queries', '', '', '', 0, 0, 0 );

    if (a<1)or(a>N)or(b<1)or(b>N)or(c<1)or(c>N)or(d<1)or(d>N) then
        die( 'Arguments must be between 1 and N', '', '', '', 0, 0, 0);

    compare_swap_ints( a, b );
    compare_swap_ints( a, c );
    compare_swap_ints( a, d );
    compare_swap_ints( b, c );
    compare_swap_ints( b, d );
    compare_swap_ints( c, d );

    if ( a = b ) or ( b = c ) or ( c = d ) then
        die( 'All 4 arguments must be distinct', '', '', '', 0, 0, 0);

    ret := (arr[b-1]+arr[c-1]) div 2;

    if LOG then
    begin
        append(flog);
        writeln( flog, ret );
        close(flog);
    end;

    Meandian := ret;
end;

procedure Solution(var sol : array of longint);
var i, expected : longint;
begin
    if LOG then
    begin
        append(flog);
        write(flog, 'Solution => ');
        close(flog);
    end;

    for i:=0 to N-1 do
    begin
        if (arr[i]=cant[0])or(arr[i]=cant[1])or
           (arr[i]=cant[2])or(arr[i]=cant[3]) then expected:=-1 else expected := arr[i];
         if ( sol[i] <> expected ) then
            die( 'Number ',' is wrong (',', should be ',')', i+1, sol[i], expected);
    end;

    (* ok - full points *)
    if LOG then
    begin
        append(flog);
        writeln(flog, 'OK! (', queries, ' queries)');
        close(flog);
        writeln('OK! (', queries, ' queries)');
    end else
    begin
        writeln('OK! (', queries, ' queries)');
    end;
    halt(0);
end;

end.
